@@ -57,6 +57,38 @@ static int do_step_n = -1;
 static int migration_version = -1;
 static char *profile_dir = NULL;
 
+/* The legacy dir is used by everything before version 29 which migrates
+ * to the new directory */
+static const char *
+legacy_default_profile_dir (void)
+{
+  static char *dir = NULL;
+  if (dir == NULL)
+    dir = g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
+  return dir;
+}
+
+static const char *
+legacy_profile_dir (void)
+{
+  static char *dir = NULL;
+  if (dir == NULL)
+    {
+      /* If this isn't actually a legacy dir it starts at a later migrating step anyway */
+      if (profile_dir != NULL)
+        dir = profile_dir;
+      else
+        dir = (char *)legacy_default_profile_dir ();
+    }
+  return dir;
+}
+
+static gboolean
+legacy_dir_is_default (void)
+{
+  return !strcmp (legacy_profile_dir (), legacy_default_profile_dir ());
+}
+
 /*
  * What to do to add new migration steps:
  *  - Bump EPHY_PROFILE_MIGRATION_VERSION in lib/ephy-profile-utils.h
@@ -68,7 +100,13 @@ typedef void (*EphyProfileMigrator) (void);
 static gboolean
 profile_dir_exists (void)
 {
-  return g_file_test (ephy_dot_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
+  if (g_file_test (ephy_profile_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+    return TRUE;
+
+  if (g_file_test (legacy_profile_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+    return TRUE;
+
+  return FALSE;
 }
 
 static void
@@ -76,7 +114,7 @@ migrate_app_desktop_file_categories (void)
 {
   GList *web_apps, *l;
 
-  web_apps = ephy_web_application_get_application_list ();
+  web_apps = ephy_web_application_get_legacy_application_list ();
 
   for (l = web_apps; l; l = l->next) {
     EphyWebApplication *app = (EphyWebApplication *)l->data;
@@ -87,7 +125,7 @@ migrate_app_desktop_file_categories (void)
 
     file = g_key_file_new ();
 
-    app_path = ephy_web_application_get_profile_directory (app->name);
+    app_path = ephy_web_application_get_profile_directory (app->id);
     desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
     g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
 
@@ -170,7 +208,7 @@ migrate_insecure_passwords (void)
    * if this migrator has already run there. This way we avoid adding a new flag
    * file to clutter the profile dir just to check if this migrator has run.
    */
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_default_profile_dir ());
   if (default_profile_migration_version >= EPHY_INSECURE_PASSWORDS_MIGRATION_VERSION) {
     LOG ("Skipping insecure password migration because default profile has already migrated");
     return;
@@ -345,14 +383,14 @@ migrate_bookmarks (void)
   xmlNodePtr child;
   xmlNodePtr root;
 
-  filename = g_build_filename (ephy_dot_dir (),
+  filename = g_build_filename (legacy_profile_dir (),
                                EPHY_BOOKMARKS_FILE,
                                NULL);
   if (g_file_test (filename, G_FILE_TEST_EXISTS))
     goto out;
   g_free (filename);
 
-  filename = g_build_filename (ephy_dot_dir (),
+  filename = g_build_filename (legacy_profile_dir (),
                                "bookmarks.rdf",
                                NULL);
   if (!g_file_test (filename, G_FILE_TEST_EXISTS))
@@ -389,7 +427,7 @@ migrate_bookmarks (void)
     g_warning ("Failed to delete %s: %s", filename, g_strerror (errno));
   g_free (filename);
 
-  filename = g_build_filename (ephy_dot_dir (),
+  filename = g_build_filename (legacy_profile_dir (),
                                "ephy-bookmarks.xml",
                                NULL);
   if (g_unlink (filename) != 0)
@@ -408,22 +446,18 @@ migrate_adblock_filters (void)
   GPtrArray *filters_array = NULL;
   GError *error = NULL;
 
-  adblock_dir = g_build_filename (ephy_dot_dir (), "adblock", NULL);
+  adblock_dir = g_build_filename (legacy_profile_dir (), "adblock", NULL);
   if (!g_file_test (adblock_dir, G_FILE_TEST_IS_DIR)) {
     g_free (adblock_dir);
     return;
   }
 
-  if (!ephy_dot_dir_is_default ()) {
-    char *default_dot_dir;
-
+  if (!legacy_dir_is_default ()) {
     /* Adblock filters rules are now shared to save space */
     ephy_file_delete_dir_recursively (adblock_dir, NULL);
     g_free (adblock_dir);
 
-    default_dot_dir = ephy_default_dot_dir ();
-    adblock_dir = g_build_filename (default_dot_dir, "adblock", NULL);
-    g_free (default_dot_dir);
+    adblock_dir = g_build_filename (legacy_default_profile_dir (), "adblock", NULL);
   }
 
   filters_filename = g_build_filename (adblock_dir, "filters.list", NULL);
@@ -488,7 +522,7 @@ migrate_initial_state (void)
 
   /* EphyInitialState no longer exists. It's just window sizes, so "migrate" it
    * by simply removing the file to avoid cluttering the profile dir. */
-  filename = g_build_filename (ephy_dot_dir (), "states.xml", NULL);
+  filename = g_build_filename (legacy_profile_dir (), "states.xml", NULL);
   file = g_file_new_for_path (filename);
 
   g_file_delete (file, NULL, &error);
@@ -508,7 +542,7 @@ migrate_permissions (void)
   char *filename;
   GFile *file;
 
-  filename = g_build_filename (ephy_dot_dir (), "hosts.ini", NULL);
+  filename = g_build_filename (legacy_profile_dir (), "hosts.ini", NULL);
   file = g_file_new_for_path (filename);
   g_free (filename);
 
@@ -516,149 +550,6 @@ migrate_permissions (void)
   g_object_unref (file);
 }
 
-static const char * const deprecated_settings[] = {
-  EPHY_PREFS_DEPRECATED_USER_AGENT,
-  EPHY_PREFS_DEPRECATED_REMEMBER_PASSWORDS,
-  EPHY_PREFS_DEPRECATED_ENABLE_SMOOTH_SCROLLING,
-};
-
-static void
-migrate_deprecated_settings (void)
-{
-  for (guint i = 0; i < G_N_ELEMENTS (deprecated_settings); i++) {
-    GVariant *value;
-
-    value = g_settings_get_value (EPHY_SETTINGS_MAIN, deprecated_settings[i]);
-    g_settings_set_value (EPHY_SETTINGS_WEB, deprecated_settings[i], value);
-    g_variant_unref (value);
-  }
-
-  g_settings_sync ();
-}
-
-static gboolean
-is_deprecated_setting (const char *setting)
-{
-  for (guint i = 0; i < G_N_ELEMENTS (deprecated_settings); i++) {
-    if (!strcmp (setting, deprecated_settings[i]))
-      return TRUE;
-  }
-
-  return FALSE;
-}
-
-static void
-migrate_settings (void)
-{
-  int default_profile_migration_version;
-
-  /* The migrator is only run for the main profile and web apps,
-   * so if the profile dir is not the default one, it's a web app.
-   * If not a web app, migrate deprecated settings.
-   */
-  if (ephy_dot_dir_is_default ()) {
-    migrate_deprecated_settings ();
-    return;
-  }
-
-  /* If it's a web app, inherit settings from the default profile. */
-  /* If the default profile hasn't been migrated yet, use the deprecated settings. */
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
-  if (default_profile_migration_version < EPHY_SETTINGS_MIGRATION_VERSION) {
-    GSettings *settings;
-
-    settings = g_settings_new_with_path (EPHY_PREFS_WEB_SCHEMA, "/org/gnome/epiphany/web/");
-
-    for (guint i = 0; i < G_N_ELEMENTS (ephy_prefs_web_schema); i++) {
-      GVariant *value;
-
-      if (is_deprecated_setting (ephy_prefs_web_schema[i]))
-        continue;
-
-      value = g_settings_get_value (settings, ephy_prefs_web_schema[i]);
-      g_settings_set_value (EPHY_SETTINGS_WEB, ephy_prefs_web_schema[i], value);
-      g_variant_unref (value);
-    }
-
-    g_object_unref (settings);
-
-    for (guint i = 0; i < G_N_ELEMENTS (deprecated_settings); i++) {
-      GVariant *value;
-
-      value = g_settings_get_value (EPHY_SETTINGS_MAIN, deprecated_settings[i]);
-      g_settings_set_value (EPHY_SETTINGS_WEB, deprecated_settings[i], value);
-      g_variant_unref (value);
-    }
-  } else
-    ephy_web_application_initialize_settings (ephy_dot_dir ());
-
-  g_settings_sync ();
-}
-
-/* Originally migrator #17, but moved to spot #27 because it uses the
- * bookmarks manager, which requires that we have already run #24.
- */
-static void
-migrate_search_engines (void)
-{
-  EphyBookmarksManager *bookmarks_manager;
-  EphySearchEngineManager *search_engine_manager;
-  GSequence *bookmarks;
-  GSequenceIter *iter;
-  GList *smart_bookmarks = NULL;
-  const char *address;
-  const char *title;
-  char *default_search_engine_address;
-  const char *default_search_engine_name = _("Search the Web");
-
-  /* Search engine settings are only used in browser mode, so no need to migrate
-   * if we are not in browser mode. */
-  if (!ephy_dot_dir_is_default ())
-    return;
-
-  bookmarks_manager = ephy_bookmarks_manager_new ();
-  search_engine_manager = ephy_search_engine_manager_new ();
-
-  default_search_engine_address = g_settings_get_string (EPHY_SETTINGS_MAIN,
-                                                         EPHY_PREFS_KEYWORD_SEARCH_URL);
-  if (default_search_engine_address != NULL) {
-      ephy_search_engine_manager_add_engine (search_engine_manager,
-                                             default_search_engine_name,
-                                             default_search_engine_address,
-                                             "");
-      ephy_search_engine_manager_set_default_engine (search_engine_manager,
-                                                     default_search_engine_name);
-      g_free (default_search_engine_address);
-  }
-
-  bookmarks = ephy_bookmarks_manager_get_bookmarks (bookmarks_manager);
-  for (iter = g_sequence_get_begin_iter (bookmarks);
-       !g_sequence_iter_is_end (iter);
-       iter = g_sequence_iter_next (iter)) {
-    EphyBookmark *bookmark;
-
-    bookmark = g_sequence_get (iter);
-    address = ephy_bookmark_get_url (bookmark);
-
-    if (strstr (address, "%s") != NULL) {
-      title = ephy_bookmark_get_title (bookmark);
-      ephy_search_engine_manager_add_engine (search_engine_manager,
-                                             title,
-                                             address,
-                                             "");
-      smart_bookmarks = g_list_append (smart_bookmarks, bookmark);
-   }
-  }
-
-  for (GList *l = smart_bookmarks; l != NULL; l = l->next)
-    ephy_bookmarks_manager_remove_bookmark (bookmarks_manager,
-                                            (EphyBookmark *)(l->data));
-
-  g_list_free (smart_bookmarks);
-  g_object_unref (bookmarks_manager);
-  g_object_unref (search_engine_manager);
-}
-
 static void
 migrate_icon_database (void)
 {
@@ -693,7 +584,7 @@ migrate_passwords_to_firefox_sync_passwords (void)
    * password schema, fields that are not taken into consideration when
    * querying passwords.
    */
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_default_profile_dir ());
   if (default_profile_migration_version >= EPHY_FIREFOX_SYNC_PASSWORDS_MIGRATION_VERSION)
     return;
 
@@ -759,7 +650,7 @@ migrate_history_to_firefox_sync_history (void)
   char *history_filename;
   const char *sql_query;
 
-  history_filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
+  history_filename = g_build_filename (legacy_profile_dir (), EPHY_HISTORY_FILE, NULL);
   if (!g_file_test (history_filename, G_FILE_TEST_EXISTS)) {
     LOG ("There is no history to migrate...");
     goto out;
@@ -821,7 +712,7 @@ migrate_passwords_add_target_origin (void)
    * Adds target_origin field to all existing records,
    * with the same value as origin.
    */
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_default_profile_dir ());
   if (default_profile_migration_version >= EPHY_TARGET_ORIGIN_MIGRATION_VERSION)
     return;
 
@@ -881,69 +772,6 @@ out:
   g_list_free_full (passwords, g_object_unref);
 }
 
-static const char * const old_sync_settings[] = {
-    EPHY_PREFS_SYNC_USER,
-    EPHY_PREFS_SYNC_TIME,
-    EPHY_PREFS_SYNC_DEVICE_ID,
-    EPHY_PREFS_SYNC_DEVICE_NAME,
-    EPHY_PREFS_SYNC_FREQUENCY,
-    EPHY_PREFS_SYNC_WITH_FIREFOX,
-    EPHY_PREFS_SYNC_BOOKMARKS_ENABLED,
-    EPHY_PREFS_SYNC_BOOKMARKS_TIME,
-    EPHY_PREFS_SYNC_BOOKMARKS_INITIAL,
-    EPHY_PREFS_SYNC_PASSWORDS_ENABLED,
-    EPHY_PREFS_SYNC_PASSWORDS_TIME,
-    EPHY_PREFS_SYNC_PASSWORDS_INITIAL,
-    EPHY_PREFS_SYNC_HISTORY_ENABLED,
-    EPHY_PREFS_SYNC_HISTORY_TIME,
-    EPHY_PREFS_SYNC_HISTORY_INITIAL,
-    EPHY_PREFS_SYNC_OPEN_TABS_ENABLED,
-    EPHY_PREFS_SYNC_OPEN_TABS_TIME
-};
-
-static void
-migrate_sync_settings_path (void)
-{
-  GSettings *deprecated_settings = ephy_settings_get ("org.gnome.Epiphany.sync.DEPRECATED");
-
-  /* Sync settings are only used in browser mode, so no need to migrate if we
-   * are not in browser mode. */
-  if (!ephy_dot_dir_is_default ())
-    return;
-
-  for (guint i = 0; i < G_N_ELEMENTS (old_sync_settings); i++) {
-    GVariant *user_value;
-
-    /* Has the setting been changed from its default? */
-    user_value = g_settings_get_user_value (deprecated_settings, old_sync_settings[i]);
-
-    if (user_value != NULL) {
-      GVariant *value;
-      const GVariantType *type;
-
-      value = g_settings_get_value (deprecated_settings, old_sync_settings[i]);
-      type = g_variant_get_type (value);
-
-      /* All double values in the old sync schema have been converted to gint64 in the new schema. */
-      if (g_variant_type_equal (type, G_VARIANT_TYPE_DOUBLE)) {
-        g_settings_set_value (EPHY_SETTINGS_SYNC, old_sync_settings[i],
-                              g_variant_new_int64 (ceil (g_variant_get_double (value))));
-      } else {
-        g_settings_set_value (EPHY_SETTINGS_SYNC, old_sync_settings[i], value);
-      }
-
-      /* We do not want to ever run this migration again, to avoid writing old
-       * values over new ones. So be cautious and reset the old settings. */
-      g_settings_reset (deprecated_settings, old_sync_settings[i]);
-
-      g_variant_unref (value);
-      g_variant_unref (user_value);
-    }
-  }
-
-  g_settings_sync ();
-}
-
 static void
 migrate_sync_device_info (void)
 {
@@ -955,7 +783,7 @@ migrate_sync_device_info (void)
   char *record;
   int default_profile_migration_version;
 
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_default_profile_dir ());
   if (default_profile_migration_version >= EPHY_SYNC_DEVICE_ID_MIGRATION_VERSION)
     return;
 
@@ -1047,7 +875,7 @@ migrate_bookmarks_timestamp (void)
   char *filename;
   int length;
 
-  filename = g_build_filename (ephy_dot_dir (), EPHY_BOOKMARKS_FILE, NULL);
+  filename = g_build_filename (legacy_profile_dir (), EPHY_BOOKMARKS_FILE, NULL);
   root_table_in = gvdb_table_new (filename, TRUE, &error);
   if (error) {
     g_warning ("Failed to create Gvdb table: %s", error->message);
@@ -1128,7 +956,7 @@ migrate_passwords_timestamp (void)
   int default_profile_migration_version;
 
   /* We want this migration to run only once. */
-  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_default_profile_dir ());
   if (default_profile_migration_version >= EPHY_PASSWORDS_TIMESTAMP_MIGRATION_VERSION)
     return;
 
@@ -1215,6 +1043,139 @@ migrate_annoyance_list (void)
   g_strfreev (modified_filters);
 }
 
+static gboolean
+move_directory_contents (GFile *source,
+                         GFile *dest)
+{
+  /* Just a sanity check as it should already exist */
+  g_file_make_directory (dest, NULL, NULL);
+
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GFileEnumerator) direnum = g_file_enumerate_children (source, G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                                                  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                                                  NULL, &error);
+  if (error) {
+    g_warning ("Failed to enumerate files: %s", error->message);
+    return FALSE;
+  }
+
+  while (TRUE) {
+    GFileInfo *info;
+    g_autoptr(GError) error = NULL;
+    if (!g_file_enumerator_iterate (direnum, &info, NULL, NULL, &error)) {
+      g_warning ("Failed to enumerate dir: %s", error->message);
+      return FALSE;
+    }
+    if (!info)
+      break;
+
+    g_autoptr(GFile) source_f = g_file_get_child (source, g_file_info_get_name (info));
+    g_autoptr(GFile) dest_f = g_file_get_child (dest, g_file_info_get_name (info));
+    if (!g_file_move (source_f, dest_f, G_FILE_COPY_NONE, NULL, NULL, NULL, &error)) {
+      g_warning ("Failed to move %s: %s", g_file_info_get_name (info), error->message);
+      return FALSE;
+    }
+  }
+
+  if (!g_file_delete (source, NULL, &error))
+    g_warning ("Failed to delete left-over source: %s", error->message);
+
+  return TRUE;
+}
+
+static void
+migrate_profile_directories (void)
+{
+  GList *web_apps, *l;
+
+  /* Web app profiles moved from config to data dirs */
+  web_apps = ephy_web_application_get_legacy_application_list ();
+  for (l = web_apps; l; l = l->next) {
+    EphyWebApplication *app = (EphyWebApplication *)l->data;
+    g_autoptr(GError) error = NULL;
+
+    g_autofree char *old_name = g_strconcat ("app-epiphany-", app->id, NULL);
+    g_autofree char *old_path = g_build_filename (legacy_profile_dir (), old_name, NULL);
+    g_autoptr(GFile) old_directory = g_file_new_for_path (old_path);
+
+    g_autofree char *app_path = ephy_web_application_get_profile_directory (app->id);
+    g_autoptr(GFile) new_directory = g_file_new_for_path (app_path);
+
+    if (!move_directory_contents (old_directory, new_directory)) {
+      continue;
+    }
+
+    // Create an empty file to indicate it's an app
+    g_autofree char *app_file = g_build_filename (app_path, ".app", NULL);
+    int fd = g_open (app_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+    if (fd < 0)
+      g_warning ("Failed to create .app file: %s", g_strerror (errno));
+    else
+      close (fd);
+
+    // Update Exec and Icon to point to the new profile dir
+    g_autofree char *old_profile_prefix = g_build_filename (legacy_profile_dir (), "app-epiphany-", NULL);
+    g_autofree char *new_profile_prefix = g_build_filename (g_get_user_data_dir (), "epiphany-", NULL);
+    g_autoptr(GKeyFile) file = g_key_file_new ();
+    g_autofree char *desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
+    g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
+
+    g_autofree char *exec = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, &error);
+    if (exec == NULL) {
+      g_warning ("Failed to get Exec key from %s: %s", desktop_file_path, error->message);
+      continue;
+    }
+    g_autofree char *new_exec = ephy_string_find_and_replace (exec, old_profile_prefix, new_profile_prefix);
+    LOG ("migrate_profile_directories: setting Exec to %s", new_exec);
+    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, new_exec);
+
+    g_autofree char *icon = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, &error);
+    if (exec == NULL) {
+      g_warning ("Failed to get Icon key from %s: %s", desktop_file_path, error->message);
+      continue;
+    }
+    g_autofree char *new_icon = ephy_string_find_and_replace (icon, old_profile_prefix, new_profile_prefix);
+    LOG ("migrate_profile_directories: setting Icon to %s", new_icon);
+    g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, new_icon);
+
+    if (!g_key_file_save_to_file (file, desktop_file_path, &error))
+      g_warning ("Failed to save desktop file %s", error->message);
+
+    g_autofree char *desktop_symlink_path = g_build_filename (g_get_user_data_dir (),
+                                                              "applications",
+                                                              app->desktop_file,
+                                                              NULL);
+    g_autoptr(GFile) desktop_symlink = g_file_new_for_path (desktop_symlink_path);
+    LOG ("Symlinking %s to %s", desktop_symlink_path, desktop_file_path);
+
+    // Try removing old symlink, failure is ok assuming it doesn't exist.
+    if (!g_file_delete (desktop_symlink, NULL, &error)) {
+      g_warning ("Failed to remove old symbolic link: %s", error->message);
+      g_clear_error (&error);
+    }
+
+    if (!g_file_make_symbolic_link (desktop_symlink, desktop_file_path, NULL, &error))
+      g_warning ("Failed to make symbolic link: %s", error->message);
+  }
+
+  ephy_web_application_free_application_list (web_apps);
+
+  /* The default profile also changed directories */
+  g_autoptr(GFile) old_directory = g_file_new_for_path (legacy_default_profile_dir ());
+  g_autoptr(GFile) new_directory = g_file_new_for_path (ephy_default_profile_dir ());
+
+  if (!move_directory_contents (old_directory, new_directory))
+    return;
+
+  /* We are also moving some cache directories so just remove the old ones */
+  g_autoptr(GFile) adblock_directory = g_file_get_child (new_directory, "adblock");
+  g_file_delete (adblock_directory, NULL, NULL);
+  g_autoptr(GFile) gsb_file = g_file_get_child (new_directory, "gsb-threats.db");
+  g_file_delete (gsb_file, NULL, NULL);
+  g_autoptr(GFile) gsb_journal_file = g_file_get_child (new_directory, "gsb-threats.db-journal");
+  g_file_delete (gsb_journal_file, NULL, NULL);
+}
+
 static void
 migrate_nothing (void)
 {
@@ -1226,6 +1187,84 @@ migrate_nothing (void)
    */
 }
 
+static void
+migrate_zoom_level (void)
+{
+  EphySQLiteConnection *history_db = NULL;
+  EphySQLiteStatement *statement = NULL;
+  GError *error = NULL;
+  char *history_filename;
+  const char *sql_query;
+
+  history_filename = g_build_filename (ephy_profile_dir (), EPHY_HISTORY_FILE, NULL);
+  if (!g_file_test (history_filename, G_FILE_TEST_EXISTS)) {
+    LOG ("There is no history to migrate...");
+    goto out;
+  }
+
+  history_db = ephy_sqlite_connection_new (EPHY_SQLITE_CONNECTION_MODE_READWRITE,
+                                           history_filename);
+  ephy_sqlite_connection_open (history_db, &error);
+  if (error) {
+    g_warning ("Failed to open history database: %s", error->message);
+    g_clear_object (&history_db);
+    goto out;
+  }
+
+  /* Update zoom level values. */
+  sql_query = "UPDATE hosts SET zoom_level = 0.0 WHERE zoom_level = 1.0";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to update zoom level: %s", error->message);
+    goto out;
+  }
+
+  sql_query = "CREATE TABLE hosts_backup ("
+              "id INTEGER PRIMARY KEY,"
+              "url LONGVARCAR,"
+              "title LONGVARCAR,"
+              "visit_count INTEGER DEFAULT 0 NOT NULL,"
+              "zoom_level REAL DEFAULT 0.0)";
+
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to create host backup table: %s", error->message);
+    goto out;
+  }
+
+  sql_query = "INSERT INTO hosts_backup SELECT id,url,title,visit_count,zoom_level FROM hosts";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to copy data from hosts to hosts_backup: %s", error->message);
+    goto out;
+  }
+
+  sql_query = "DROP TABLE hosts";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to remove table hosts: %s", error->message);
+    goto out;
+  }
+
+  sql_query = "ALTER TABLE hosts_backup RENAME TO hosts";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to rename hosts_backup to hosts: %s", error->message);
+    goto out;
+  }
+
+out:
+  g_free (history_filename);
+  if (history_db) {
+    ephy_sqlite_connection_close (history_db);
+    g_object_unref (history_db);
+  }
+  if (statement)
+    g_object_unref (statement);
+  if (error)
+    g_error_free (error);
+}
+
 /* If adding anything here, you need to edit EPHY_PROFILE_MIGRATION_VERSION
  * in ephy-profile-utils.h. */
 const EphyProfileMigrator migrators[] = {
@@ -1244,19 +1283,21 @@ const EphyProfileMigrator migrators[] = {
   /* 13 */ migrate_adblock_filters,
   /* 14 */ migrate_initial_state,
   /* 15 */ migrate_permissions,
-  /* 16 */ migrate_settings,
+  /* 16 */ migrate_nothing,
   /* 17 */ migrate_nothing,
   /* 18 */ migrate_icon_database,
   /* 19 */ migrate_passwords_to_firefox_sync_passwords,
   /* 20 */ migrate_history_to_firefox_sync_history,
   /* 21 */ migrate_passwords_add_target_origin,
-  /* 22 */ migrate_sync_settings_path,
+  /* 22 */ migrate_nothing,
   /* 23 */ migrate_sync_device_info,
   /* 24 */ migrate_bookmarks_timestamp,
   /* 25 */ migrate_passwords_timestamp,
   /* 26 */ migrate_nothing,
-  /* 27 */ migrate_search_engines,
-  /* 28 */ migrate_annoyance_list
+  /* 27 */ migrate_nothing,
+  /* 28 */ migrate_annoyance_list,
+  /* 29 */ migrate_zoom_level,
+  /* 30 */ migrate_profile_directories,
 };
 
 static gboolean
@@ -1287,7 +1328,8 @@ ephy_migrator (void)
     return TRUE;
   }
 
-  latest = ephy_profile_utils_get_migration_version ();
+  latest = MAX (ephy_profile_utils_get_migration_version (),
+                ephy_profile_utils_get_migration_version_for_profile_dir (legacy_profile_dir ()));
 
   LOG ("Running migrators up to version %d, current migration version is %d.",
        EPHY_PROFILE_MIGRATION_VERSION, latest);
